/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void gen_crc_table( void );
unsigned short crc_gen( unsigned short crc_init, unsigned long crc_in, uint8_t ind1, uint8_t BitOrder );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void CRC_Configure( void );
void CRC_CRC16Run( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// CRC Polynomial
#define CRC8               0x07
#define CRC16              0x8005
#define CRC16_CCITT        0x1021
#define CRC32              0x04c11db7

// Input Data Size
#define WORD_LENGTH        0
#define BUF_LENGTH         1


// Bit Order
#define CRC_MSB_FIRST      0
#define CRC_LSB_FIRST      1

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "CRC_CRC16 Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: CRC \n\r"
                        "CRC Initial Data:  CRC_INIT = 0 \n\r"
                        "CRC Input Data:    CRC_IN   = 0x98765432 \n\r"
                        "************************************************\n\r\n\r";

unsigned short    polynominal = CRC16;
unsigned long     crc_start_adr;
unsigned long     crc_end_adr;
unsigned short    crc_table[256];

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         gen_crc_table
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void gen_crc_table( void )
{
   unsigned short    i, j;
   unsigned short    crc_accum;

   for( i = 0; i < 256; i++ )
   {
      crc_accum = i << 8;
      for( j = 0; j < 8; j++ )
      {
         if( crc_accum & 0x8000 )
         {
            crc_accum = ( crc_accum <<  1 ) ^ polynominal;
         }
         else
         {
            crc_accum <<= 1;
         }
      }
      crc_table[i] = crc_accum;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         crc_gen
 * @param[in]     crc_init
 * @param[in]     crc_in
 * @param[in]     ind1
 * @param[in]     BitOrder
 * @return        CRC Result
 *//*-------------------------------------------------------------------------*/
unsigned short crc_gen( unsigned short crc_init, unsigned long crc_in, uint8_t ind1, uint8_t BitOrder )
{
   uint8_t           i;
   unsigned long     tmp = 0;
   unsigned long*     st_adr, *ed_adr;

   gen_crc_table();
   if( ind1 == WORD_LENGTH )
   {
      if( BitOrder != CRC_MSB_FIRST )
      {
         for( i = 0 ; i < 32 ; i++ )
         {
            tmp <<= 1;
            if( crc_in & 0x00000001ul )
            {
               tmp |= 0x00000001;
            }
            crc_in >>= 1;
         }
         crc_in = tmp;
      }
      for( i = 0 ; i < 4 ; i++ )
      {
         crc_init = ( crc_init <<  8 ) ^ crc_table[( ( crc_init >> 8 ) ^ ( unsigned short )( crc_in >> ( 24 - i * 8 ) ) ) & 0x00FF];
      }
   }
   else
   {
      st_adr = ( unsigned long* )crc_start_adr;
      ed_adr = ( unsigned long* )crc_end_adr;
      while( st_adr <= ed_adr )
      {
         crc_in = *st_adr++;
         if( BitOrder != CRC_MSB_FIRST )
         {
            for( i = 0 ; i < 32 ; i++ )
            {
               tmp <<= 1;
               if( crc_in & 0x00000001ul )
               {
                  tmp |= 0x00000001;
               }
               crc_in >>= 1;
            }
            crc_in = tmp;
         }
         for( i = 0 ; i < 4 ; i++ )
         {
            crc_init = ( crc_init <<  8 ) ^ crc_table[( ( crc_init >> 8 ) ^ ( unsigned short )( crc_in >> ( 24 - i * 8 ) ) ) & 0x00FF];
         }
      }
   }
   return crc_init;
}

/*-------------------------------------------------------------------------*//**
 * @brief         CRC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CRC_Configure( void )
{
   // init crc
   HAL_CRC_Init();
}

/*-------------------------------------------------------------------------*//**
 * @brief         CRC_CRC16Run
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CRC_CRC16Run( void )
{
   uint16_t    crc16_init;
   uint32_t    crc16_in;
   uint16_t    crc16;

   // init
   crc16_init = 0;            // CRC_INIT Value
   crc16_in = 0x98765432;     // CRC_IN Value

   // Manual Calculation
   crc16 = crc_gen( crc16_init, crc16_in, WORD_LENGTH, CRC_MSB_FIRST );
   _DBG( "[Manual Calculation Result] crc16 = 0x" );
   _DBH16( crc16 );
   _DBG( "\n\r" );

   // CRC Block Calculation
   CRC->INIT = crc16_init;
   CRC->CR = 0
             | RLTCLR_Init          // CRC/Checksum Result Data Register (CRC_RLT) Initialization
             | MODS_UserMode        // User/Auto Mode Selection
             | MDSEL_CRC            // CRC/Checksum Selection
             | POLYS_CRC16          // Polynomial Selection (CRC only)
             | SARINC_Disable       // CRC/Checksum Start Address Auto Increment Control (User mode only)
             | FIRSTBS_msbFirst     // First Shifted-in Selection (CRC only)
             | INSIZE_32Bit         // Input Data Size Selection
             ;
   CRCRun();
   CRC->IN = crc16_in;
   crc16 = CRC->RLT;
   CRCStop();
   _DBG( "[CRC Block Calculation Result] crc16 = 0x" );
   _DBH16( crc16 );
   _DBG( "\n\r" );

   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* CRC_Configure */
   CRC_Configure();

   /* CRC_CRC16Run */
   CRC_CRC16Run();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

