/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

unsigned long crc_gen( unsigned long crc_init, unsigned char InputDataSize, unsigned char BitOrder );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void CRC_Configure( void );
void CRC_CRC8Run( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// CRC Polynomial
#define CRC8               0x07
#define CRC16 			      0x8005
#define CRC16_CCITT 		   0x1021
#define CRC32 			      0x04c11db7

// Input Data Size
#define WORD_LENGTH        0
#define HALF_WORD_LENGTH   1
#define BYTE_LENGTH        2

// Bit Order
#define CRC_MSB_FIRST      0
#define CRC_LSB_FIRST      1

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "CRC_CRC8 Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: CRC \n\r"
                        "CRC Initial Data:  CRC_INIT = 0 \n\r"
                        "CRC Input Data:    CRC_IN   = 0x98765432 \n\r"
                        "************************************************\n\r\n\r";

unsigned long     polynominal = CRC8;
unsigned long     crc_start_adr;
unsigned long     crc_end_adr;
unsigned char     crc_in_comp = 0;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         crc_gen
 * @param[in]     crc_init
 * @param[in]     InputDataSize
 * @param[in]     BitOrder
 * @return        CRC Result
 *//*-------------------------------------------------------------------------*/
unsigned long crc_gen( unsigned long crc_init, unsigned char InputDataSize, unsigned char BitOrder )
{
   unsigned char     i;
   unsigned long     tmp = 0;
   unsigned long     CRC_MSB;
   unsigned long     crc_in;
   unsigned long*    st_adr_32bit;
   unsigned long*    ed_adr_32bit;
   unsigned short*   st_adr_16bit;
   unsigned short*   ed_adr_16bit;
   unsigned char*    st_adr_8bit;
   unsigned char*    ed_adr_8bit;

   // get crc msb mask
   if( polynominal == CRC8 )
   {
      CRC_MSB = 0x0000080;
   }
   else if( ( polynominal == CRC16 ) || ( polynominal == CRC16_CCITT ) )
   {
      CRC_MSB = 0x00008000;
   }
   else if( polynominal == CRC32 )
   {
      CRC_MSB = 0x80000000;
   }

   // word size calculation
   if( InputDataSize == WORD_LENGTH )
   {
      st_adr_32bit = ( unsigned long* )crc_start_adr;
      ed_adr_32bit = ( unsigned long* )crc_end_adr;

      while( st_adr_32bit <= ed_adr_32bit )
      {
         crc_in = *st_adr_32bit++;
         if( BitOrder == CRC_LSB_FIRST )
         {
            for( i = 0 ; i < 32 ; i++ )
            {
               tmp <<= 1;
               if( crc_in & 0x00000001ul )
               {
                  tmp |= 0x00000001;
               }
               crc_in >>= 1;
            }
            crc_in = tmp;
         }
         if( crc_in_comp )
         {
            crc_in = crc_in ^ 0xFFFFFFFF;   // if INCOMP=1 ? (1's complement)
         }

         for( i = 0 ; i < 32 ; i++ )
         {
            tmp = crc_init;
            crc_init <<= 1;
            if( tmp & CRC_MSB )
            {
               crc_init ^= polynominal;   // if MSB(of crc_init) = 1 (
            }

            // when polynominal is 32bits, MSB is bit [31]
            // when polynominal is 16bits, MSB is bit [15]
            // when polynominal is 8bits, MSB is bit [7]
            if( crc_in & 0x80000000 )
            {
               crc_init ^= polynominal;   // if input 1-bit(MSB of 32bit(data input size)) = 1 ?
            }
            crc_in <<= 1;
         }
      }
   }

   // half word size calculation
   else if( InputDataSize == HALF_WORD_LENGTH )
   {
      st_adr_16bit = ( unsigned short* )crc_start_adr;
      ed_adr_16bit = ( unsigned short* )crc_end_adr;

      while( st_adr_16bit <= ed_adr_16bit )
      {
         crc_in = *st_adr_16bit++;
         if( BitOrder == CRC_LSB_FIRST )
         {
            for( i = 0 ; i < 16 ; i++ )
            {
               tmp <<= 1;
               if( crc_in & 0x00000001ul )
               {
                  tmp |= 0x00000001;
               }
               crc_in >>= 1;
            }
            crc_in = tmp;
         }
         if( crc_in_comp )
         {
            crc_in = crc_in ^ 0x0000FFFF;   // if INCOMP=1 ? (1's complement)
         }

         for( i = 0 ; i < 16 ; i++ )
         {
            tmp = crc_init;
            crc_init <<= 1;
            if( tmp & CRC_MSB )
            {
               crc_init ^= polynominal;   // if MSB(of crc_init) = 1 (
            }

            // when polynominal is 32bits, MSB is bit [31]
            // when polynominal is 16bits, MSB is bit [15]
            // when polynominal is 8bits, MSB is bit [7]
            if( crc_in & 0x00008000 )
            {
               crc_init ^= polynominal;   // if input 1-bit(MSB of 16bit(data input size)) = 1 ?
            }
            crc_in <<= 1;
         }
      }
   }

   // byte size calculation
   else if( InputDataSize == BYTE_LENGTH )
   {
      st_adr_8bit = ( unsigned char* )crc_start_adr;
      ed_adr_8bit = ( unsigned char* )crc_end_adr;

      while( st_adr_8bit <= ed_adr_8bit )
      {
         crc_in = *st_adr_8bit++;
         if( BitOrder == CRC_LSB_FIRST )
         {
            for( i = 0 ; i < 8 ; i++ )
            {
               tmp <<= 1;
               if( crc_in & 0x00000001ul )
               {
                  tmp |= 0x00000001;
               }
               crc_in >>= 1;
            }
            crc_in = tmp;
         }
         if( crc_in_comp )
         {
            crc_in = crc_in ^ 0x000000FF;   // if INCOMP=1 ? (1's complement)
         }

         for( i = 0 ; i < 8 ; i++ )
         {
            tmp = crc_init;
            crc_init <<= 1;
            if( tmp & CRC_MSB )
            {
               crc_init ^= polynominal;   // if MSB(of crc_init) = 1 (
            }

            // when polynominal is 32bits, MSB is bit [31]
            // when polynominal is 16bits, MSB is bit [15]
            // when polynominal is 8bits, MSB is bit [7]
            if( crc_in & 0x00000080 )
            {
               crc_init ^= polynominal;   // if input 1-bit(MSB of 8bit(data input size)) = 1 ?
            }
            crc_in <<= 1;
         }
      }
   }

   return crc_init;
}

/*-------------------------------------------------------------------------*//**
 * @brief         CRC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CRC_Configure( void )
{
   // init crc
   HAL_CRC_Init();
}

/*-------------------------------------------------------------------------*//**
 * @brief         CRC_CRC8Run
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CRC_CRC8Run( void )
{
   uint8_t     crc8_init;
   uint32_t    crc8_in;
   uint8_t     crc8;

   // init
   crc8_init = 0;             // CRC_INIT Value
   crc8_in = 0x98765432;      // CRC_IN Value

   // Manual Calculation
   crc_start_adr = ( unsigned long )&crc8_in;
   crc_end_adr   = ( unsigned long )&crc8_in;
   crc8 = crc_gen( crc8_init, WORD_LENGTH, CRC_MSB_FIRST );
   _DBG( "[Manual Calculation Result] crc8 = 0x" );
   _DBH( crc8 );
   _DBG( "\n\r" );

   // CRC Block Calculation
   CRC->INIT = crc8_init;
   CRC->CR = 0
             | RLTCLR_Init          // CRC/Checksum Result Data Register (CRC_RLT) Initialization
             | MODS_UserMode        // User/Auto Mode Selection
             | MDSEL_CRC            // CRC/Checksum Selection
             | POLYS_CRC8           // Polynomial Selection (CRC only)
             | SARINC_Disable       // CRC/Checksum Start Address Auto Increment Control (User mode only)
             | FIRSTBS_msbFirst     // First Shifted-in Selection (CRC only)
             | INSIZE_32Bit         // Input Data Size Selection
             ;
   CRCRun();
   CRC->IN = crc8_in;
   crc8 = CRC->RLT;
   CRCStop();
   _DBG( "[CRC Block Calculation Result] crc8 = 0x" );
   _DBH( crc8 );
   _DBG( "\n\r" );

   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* CRC_Configure */
   CRC_Configure();

   /* CRC_CRC8Run */
   CRC_CRC8Run();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

