/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void CRC_Configure( void );
void CRC_ChecksumRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "CRC_Checksum Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: CRC \n\r"
                        "CRC Initial Data:  CRC_INIT = 0 \n\r"
                        "CRC Input Data:    CRC_IN   = 0x12345678 \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         CRC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CRC_Configure( void )
{
   // init crc
   HAL_CRC_Init();
}

/*-------------------------------------------------------------------------*//**
 * @brief         CRC_ChecksumRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CRC_ChecksumRun( void )
{
   uint32_t    i;
   uint16_t    chksum_init;
   uint8_t     chksum_in[4];
   uint16_t    chksum;
   uint32_t    tmp;

   // init
   chksum_init = 0;           // CRC_INIT Value
   chksum_in[0] = 0x12;       // CRC_IN Value
   chksum_in[1] = 0x34;       // CRC_IN Value
   chksum_in[2] = 0x56;       // CRC_IN Value
   chksum_in[3] = 0x78;       // CRC_IN Value

   // Manual Calculation
   chksum = chksum_init;
   for( i = 0; i < 4; i++ )
   {
      chksum += chksum_in[i];
   }
   _DBG( "[Manual Calculation Result] chksum = 0x" );
   _DBH16( chksum );    // chksum = 0x0114
   _DBG( "\n\r" );

   // CRC Block Calculation
   CRC->INIT = chksum_init;
   CRC->CR = 0
             | RLTCLR_Init          // CRC/Checksum Result Data Register (CRC_RLT) Initialization
             | MODS_UserMode        // User/Auto Mode Selection
             | MDSEL_Checksum       // CRC/Checksum Selection
             // Polynomial Selection (CRC only)
             | SARINC_Disable       // CRC/Checksum Start Address Auto Increment Control (User mode only)
             // First Shifted-in Selection (CRC only)
             | INSIZE_8Bit          // Input Data Size Selection
             ;
   CRCRun();
   CRC->IN = chksum_in[0];
   CRC->IN = chksum_in[1];
   CRC->IN = chksum_in[2];
   CRC->IN = chksum_in[3];
   chksum = CRC->RLT;
   CRCStop();
   _DBG( "[CRC Block Calculation Result] chksum = 0x" );
   _DBH16( chksum );    // chksum = 0x0114
   _DBG( "\n\r" );
   delay();

   // Auto Calculation
   //    Flash 64KB Start Address 0x1000_0000, End Address 0x1000_FFFF
   HAL_CRC_SetAddress( 0x10000000uL, 0x1000FFFFuL, chksum_init );
   chksum = HAL_CRC_ConfigAutoMode( MDSEL_Checksum, 0, 0, INSIZE_8Bit, INCOMP_Disable );
   _DBG( "[Auto Calculation Result] 64KB chksum = 0x" );
   _DBH16( chksum );
   _DBG( "\n\r" );

   // Auto Calculation Verify
   tmp = 0;
   for( i = 0 ; i < 0x00010000 ; i++ )
   {
      tmp += ( ( *( uint32_t* )( i ) ) & 0xff );
   }
   _DBG( "Verify chksum = 0x" );
   _DBH32( tmp );
   if( chksum == ( tmp & 0xFFFFuL ) )
   {
      _DBG( " => Matched! \n\r" );
   }
   else
   {
      _DBG( " => MisMatched! \n\r" );
   }

   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* CRC_Configure */
   CRC_Configure();

   /* CRC_ChecksumRun */
   CRC_ChecksumRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

