/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DMAC4_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void ADC_Configure( void );
void DMAC4_Configure( void );
void DMACn_ADCRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// buffer size
#define BUF_SIZE     4

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "DMACn_ADC Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: DMAC4 / ADC \n\r"
                        "ADC Input Pin:     PA1(AN1) PA4(AN4) PA7(AN7) PA8(AN8) \n\r"
                        "************************************************\n\r\n\r";

ADC_CFG_Type      ADC_Config;

// transfer complete flag
volatile Bool     TransferComplete = FALSE;

// transfer error flag
volatile Bool     TransferError = FALSE;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC4_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC4_IRQHandler_IT( void )
{
   // if dma transfer complete interrupt
   if( DMAC4->IESR_b.TRCIFGn == 1 )      // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferComplete = TRUE;

      // reset TRCIFGn
      DMAC4->IESR = DMAC4->IESR
                    &  ~DMACn_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMACn_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }

   // if dma transfer error interrupt
   if( DMAC4->IESR_b.TRERIFGn == 1 )      // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferError = TRUE;

      // reset TRERIFGn
      DMAC4->IESR = DMAC4->IESR
                    &  ~DMACn_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMACn_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_Configure( void )
{
   // ADC configuration
   {
      // adc input pin configuration
      {
         // print
         _DBG( "PA1 PA4 PA7 PA8 was configured as a AN1 AN4 AN7 AN8. \n\r\n\r" );

         // configure PA1 as a AN1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 1, 4 );

         // configure PA4 as a AN4
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 4, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 4, 4 );

         // configure PA7 as a AN7
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 7, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 7, 4 );

         // configure PA8 as a AN8
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 8, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 8, 4 );
      }

      // adc configuration
      {
         ADC_Config.TrgSel = ADC_CR_TRIG_ADST;
         ADC_Config.ExtTrgPolSel = ADC_CR_ETRGP_Disable;
         ADC_Config.ConvModSel = ADC_CR_MDSEL_Sequential;
         ADC_Config.InClkDiv = 1;         // 32M/2 = 16M
         ADC_Config.SamplingClk = 2;      // Sample&Hold: n+2 = 4 cycles
         if( HAL_ADC_Init( ADC, &ADC_Config ) != HAL_OK )
         {
            /* Initialization Error */
            Error_Handler();
         }
         HAL_ADC_ClearStatus( ADC, ADC_INT_STATUS_ALL );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC4_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC4_Configure( void )
{
   // DMAC4 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC4, PERSEL_ADC, DIR_PeriToMem, SIZE_16bit, ERFGSTP_Enable );

         // enable interrupt
         DMAC4->IESR_b.TRCIENn = 1;       // TRCIENn[04:04]   [0] in [0(disable transfer complete interrupt) 1(enable)]
         DMAC4->IESR_b.TRERIENn = 1;      // TRERIENn[05:05]   [0] in [0(disable transfer error interrupt) 1(enable)]
      }

      // unmask interrupt
      NVIC_SetPriority( DMAC4_IRQn, 3 );
      NVIC_EnableIRQ( DMAC4_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_DMAC4 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMACn_ADCRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMACn_ADCRun( void )
{
   uint16_t    adc_result[BUF_SIZE];
   uint32_t    tmp, tmp_flag;

   /* Infinite loop */
   while( 1 )
   {
      // arm rx transfer
      HAL_DMAC_Setup( ( DMACn_Type* )DMAC4, ( uint32_t )adc_result, BUF_SIZE );

      HAL_ADC_Start( ADC );                                                // Start

      //------------------
      while( !TransferComplete && !TransferError ) {}
      if( TransferComplete )
      {
         TransferComplete = FALSE;

         if( ADCInt_GetEOSFg() )
         {
            ADCInt_ClrEOSFg();

            _DBG( "AN1, AN4, AN7, AN8 Result Data: " );
            _DBH16( adc_result[0] );
            _DBG( ", " );
            _DBH16( adc_result[1] );
            _DBG( ", " );
            _DBH16( adc_result[2] );
            _DBG( ", " );
            _DBH16( adc_result[3] );
            _DBG( ".\n\r" );
            tmp_flag = FALSE;
         }
         else
         {
            _DBG( "ADC: Error occurs. So, Resetting...\n\r" );
            tmp_flag = TRUE;
         }
      }
      else
      {
         TransferError = FALSE;

         _DBG( "DAM4 for ADC: Error occurs. So, Resetting...\n\r" );
         tmp_flag = TRUE;
      }

      if( tmp_flag )
      {
         // Resetting
         // DMA
         HAL_DMAC_Init( ( DMACn_Type* )DMAC4, PERSEL_ADC, DIR_PeriToMem, SIZE_16bit, ERFGSTP_Enable );
         DMAC4->IESR_b.TRCIENn = 1;       // TRCIENn[04:04]   [0] in [0(disable transfer complete interrupt) 1(enable)]
         DMAC4->IESR_b.TRERIENn = 1;      // TRERIENn[05:05]   [0] in [0(disable transfer error interrupt) 1(enable)]

         // ADC
         if( HAL_ADC_Init( ADC, &ADC_Config ) != HAL_OK )
         {
            /* Initialization Error */
            Error_Handler();
         }
         HAL_ADC_ClearStatus( ADC, ADC_INT_STATUS_ALL );
         HAL_ADC_OvSamplingConfig( ADC, ADC_8_OV_SAMPLING, ADC_OVSCR_OVSHT_ShiftRight3Bit );
         HAL_ADC_ChannelSel( ADC, ADC_AN1 | ADC_AN4 | ADC_AN7 | ADC_AN8 );
         HAL_ADC_RdyCmd( ADC );
      }

      for( tmp = 0; tmp < 1000000; tmp++ );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* ADC_Configure */
   ADC_Configure();

   /* DMAC4_Configure */
   DMAC4_Configure();

   HAL_ADC_OvSamplingConfig( ADC, ADC_8_OV_SAMPLING, ADC_OVSCR_OVSHT_ShiftRight3Bit );     // Oversampling ratio, Data shift right
   HAL_ADC_ChannelSel( ADC, ADC_AN1 | ADC_AN4 | ADC_AN7 | ADC_AN8 );    // select channels

   HAL_ADC_RdyCmd( ADC );     // Ready for conversion

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* DMACn_ADCRun */
   DMACn_ADCRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

