/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void FlashMem_Do_PageWt( uint32_t tPageAddr, uint32_t* tBuf );
void FlashMem_Do_PageEr( uint32_t tPageAddr );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void FMC_Configure( void );
void FMC_WriteEraseRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "FMC_Flash Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: FMC \n\r"
                        "GPIO Input Pin:    PA0 \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         FlashMem_Do_PageWt
 * @param[in]     tPageAddr
 * @param[in]     tBuf
 * @return        None
 *//*-------------------------------------------------------------------------*/
void FlashMem_Do_PageWt( uint32_t tPageAddr, uint32_t* tBuf )
{
   uint32_t    UserID_Buf;

   UserID_Buf = 0x4F17DC86;
   HAL_FMC_PageWrite( UserID_Buf, tPageAddr, tBuf );
}

/*-------------------------------------------------------------------------*//**
 * @brief         FlashMem_Do_PageEr
 * @param[in]     tPageAddr
 * @return        None
 *//*-------------------------------------------------------------------------*/
void FlashMem_Do_PageEr( uint32_t tPageAddr )
{
   uint32_t    UserID_Buf;

   UserID_Buf = 0xA901358F;
   HAL_FMC_PageErase( UserID_Buf, tPageAddr );
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio input pin configuration
   {
      // print
      _DBG( "PA0 was configured as a GPIO Input Mode. \n\r\n\r" );

      // configure PA0 as a Input Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 0, INPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 0, PUPDx_EnablePU );
      HAL_GPIO_SetDebouncePin( ( Pn_Type* )PA, 0, DBCLK_HCLK1024 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         FMC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void FMC_Configure( void )
{
   // enable peripheral clock
   HAL_SCU_Peripheral_EnableClock2( PPCLKEN2_FMCLKE, PPxCLKE_Enable );
}

/*-------------------------------------------------------------------------*//**
 * @brief         FMC_WriteEraseRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void FMC_WriteEraseRun( void )
{
   uint32_t    i, keyin = 0;
   uint32_t    buf[32] =
   {
      0x55555555, 0x12345678, 0x9ABCDEF0, 0xAAAAAAAA, 0x89ABCDEF, 0x76543210, 0x44444444, 0x77777777,
      0x55555555, 0x12345678, 0x9ABCDEF0, 0xAAAAAAAA, 0x89ABCDEF, 0x76543210, 0x44444444, 0x77777777,
      0x55555555, 0x12345678, 0x9ABCDEF0, 0xAAAAAAAA, 0x89ABCDEF, 0x76543210, 0x44444444, 0x77777777,
      0x55555555, 0x12345678, 0x9ABCDEF0, 0xAAAAAAAA, 0x89ABCDEF, 0x76543210, 0x44444444, 0x77777777
   };
   uint32_t    flash_area,    *flash_result;
   uint32_t    config_area,   *config_result;

   // init for fmc control
   flash_area = 0x00007800;
   flash_result = ( uint32_t* )flash_area;
   config_area = 0x1FFFF400;
   config_result = ( uint32_t* )config_area;

   // print
   for( i = 0; i < 4; i++ )
   {
      _DBG( "Flash Write Value [" );
      _DBD32( i * 4 );
      _DBG( "] = " );
      _DBH32( buf[i] );
      _DBG( "\n\r" );
   }
   _DBG( "\n\r" );
   _DBG( "Press PA0. \n\r" );

   /* Infinite loop */
   while( 1 )
   {
      // key Input
      for( i = 0; i < 500; i++ )
      {
         if( !PA->INDR_b.INDR0 )
         {
            keyin = 1;
         }
      }

      // if key input is ok
      if( keyin == 1 )
      {
         // clear keyin
         keyin = 0;

         // flash area write/check/erase
         FlashMem_Do_PageWt( flash_area, buf );
         for( i = 0; i < 4; i++ )
         {
            _DBG( "Flash Write Result [" );
            _DBH32( ( flash_area + ( i * 4 ) ) );
            _DBG( "] = " );
            _DBH32( flash_result[i] );
            _DBG( "\n\r" );
         }
         _DBG( "\n\r" );
         FlashMem_Do_PageEr( flash_area );

         // confige area write/check/erase
         FlashMem_Do_PageWt( config_area, buf );
         for( i = 0; i < 4; i++ )
         {
            _DBG( "Flash Write Result [" );
            _DBH32( ( config_area + ( i * 4 ) ) );
            _DBG( "] = " );
            _DBH32( config_result[i] );
            _DBG( "\n\r" );
         }
         _DBG( "\n\r" );
         FlashMem_Do_PageEr( config_area );

         _DBG( "Press PA0. \n\r" );
      }

      // delay
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* FMC_Configure */
   FMC_Configure();

   /* FMC_WriteEraseRun */
   FMC_WriteEraseRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

