/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void Buffer_Init( uint8_t* buffer, uint8_t type );
void App_I2C_Polling_Master( void );
void App_I2C_Interrupt_Master( void );
void App_I2C_Polling_Slave( void );
void App_I2C_Interrupt_Slave( void );
void I2C1_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void I2C1_Configure( void );
void I2Cn_MasterSlaveRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// Own Slave address in Slave I2C device
#define I2CDEV_S_ADDR      (0xC0 >> 1)

// Max buffer length
#define BUFFER_SIZE        10

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "I2Cn_MasterSlave Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: I2C1 \n\r"
                        "I2C SCL Pin:       PB6(SCL1) \n\r"
                        "I2C SDA Pin:       PB7(SDA1) \n\r"
                        "Press 1 ~ 4 to select I2C Operation Mode: \n\r"
                        "  -  1: I2C Polling   Master \n\r"
                        "  -  2: I2C Interrupt Master \n\r"
                        "  -  3: I2C Polling   Slave \n\r"
                        "  -  4: I2C Interrupt Slave \n\r"
                        "************************************************\n\r\n\r";

const uint8_t menu1[] = " - Press x to exit this mode! \n\r";

const uint8_t menu3[] = " - Press c to continue... \n\r";

Bool        complete;
Bool        isMasterMode = TRUE;

// These global variables below used in interrupt mode - Slave device
uint8_t     Master_Buf[BUFFER_SIZE];

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         Initializes buffer
 * @param[in]     buffer
 *                   buffer to initialize
 * @param[in]     type:
 *                   -  0:    Initialize buffer with 0
 *                   -  1:    Initialize buffer with increment value from 0
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Buffer_Init( uint8_t* buffer, uint8_t type )
{
   uint8_t     i;

   if( type )
   {
      for( i = 0; i < BUFFER_SIZE; i++ )
      {
         buffer[i] = i;
      }
   }
   else
   {
      for( i = 0; i < BUFFER_SIZE; i++ )
      {
         buffer[i] = 0;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2C master application in polling mode
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void App_I2C_Polling_Master( void )
{
   I2Cn_M_SETUP_Type    transferMCfg;
   uint8_t              buffer = 0xFF, regAddr = 0x00;
   uint8_t              Master_Buf[BUFFER_SIZE];

   while( 1 )
   {
      _DBG( "I2Cn Polling Master is running... \n\r" );
      _DBG( menu1 );
      _DBG( menu3 );

      while( 1 )
      {
         buffer = _DG();
         if( buffer == 'x' || buffer == 'X' )
         {
            return;
         }
         else if( buffer == 'c' || buffer == 'C' )
         {
            break;
         }
      }

      // Transmit --------------------------------------------------------

      _DBG( "Press '1' to transmit \n\r" );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == '1' )
         {
            break;
         }
      }
      _DBG( "Transmitting... \n\r" );

      // Initialize buffer
      Buffer_Init( Master_Buf, 0 );
      Master_Buf[0] = 0xa1; //address
      Master_Buf[1] = 0xa2; //data
      Master_Buf[2] = 0xa4; //data
      Master_Buf[3] = 0xa6; //data
      Master_Buf[4] = 0xa8; //data
      Master_Buf[5] = 0xaA; //data
      transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
      transferMCfg.tx_data = Master_Buf;
      transferMCfg.tx_length = 6;
      HAL_I2C_Master_Transmit( ( I2Cn_Type* )I2C1, &transferMCfg, I2Cn_TRANSFER_POLLING );

      // Transmit and receive --------------------------------------------------------

      _DBG( "Press '2' to Transmit, then repeat start and receive... \n\r" );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == '2' )
         {
            break;
         }
      }

      // Initialize buffer
      Buffer_Init( Master_Buf, 0 );
      transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
      transferMCfg.tx_data = &regAddr;
      transferMCfg.tx_length = 1;
      transferMCfg.rx_data = Master_Buf;
      transferMCfg.rx_length = 5;
      transferMCfg.tx_count = 0;
      transferMCfg.rx_count = 0;
      HAL_I2C_MasterTransferData( ( I2Cn_Type* )I2C1, &transferMCfg, I2Cn_TRANSFER_POLLING );
      NOP();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2C master application using interrupt
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void App_I2C_Interrupt_Master( void )
{
   I2Cn_M_SETUP_Type    transferMCfg;
   uint8_t              buffer = 0xFF, regAddr = 0;
   uint8_t              Master_Buf[BUFFER_SIZE];

   while( 1 )
   {
      _DBG( "I2Cn Interrupt Master is running... \n\r" );
      _DBG( menu1 );
      _DBG( menu3 );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == 'x' || buffer == 'X' )
         {
            return;
         }
         else if( buffer == 'c' || buffer == 'C' )
         {
            break;
         }
      }

      // Transmit --------------------------------------------------------

      _DBG( "Press '1' to transmit \n\r" );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == '1' )
         {
            break;
         }
      }
      _DBG( "Transmitting... \n\r" );

      // Initialize buffer
      complete = FALSE;
      Buffer_Init( Master_Buf, 0 );
      Master_Buf[0] = 0x1A; //address
      Master_Buf[1] = 0x2A; //data
      Master_Buf[2] = 0x4A; //data
      Master_Buf[3] = 0x6A; //data
      Master_Buf[4] = 0x8A; //data
      Master_Buf[5] = 0xA5; //data
      transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
      transferMCfg.tx_data = Master_Buf;
      transferMCfg.tx_length = 6;
      HAL_I2C_Master_Transmit( ( I2Cn_Type* )I2C1, &transferMCfg, I2Cn_TRANSFER_INTERRUPT );
      while( !complete );

      // Transmit and receive --------------------------------------------------------

      _DBG( "Press '2' to Transmit, then repeat start and receive... \n\r" );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == '2' )
         {
            break;
         }
      }

      // Initialize buffer
      complete = FALSE;
      Buffer_Init( Master_Buf, 0 );
      transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
      transferMCfg.tx_data = &regAddr;
      transferMCfg.tx_length = 1;
      transferMCfg.rx_data = Master_Buf;
      transferMCfg.rx_length = 5;
      transferMCfg.tx_count = 0;
      transferMCfg.rx_count = 0;
      HAL_I2C_MasterTransferData( ( I2Cn_Type* )I2C1, &transferMCfg, I2Cn_TRANSFER_INTERRUPT );
      while( !complete );
      NOP();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2C slave application in polling mode
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void App_I2C_Polling_Slave( void )
{
   I2Cn_S_SETUP_Type    transferSCfg;
   uint8_t              buffer = 0xFF;
   uint8_t              Slave_Buf[BUFFER_SIZE], S_buff[2];
   uint8_t              i = 0;

   while( 1 )
   {
      _DBG( "I2Cn Polling Slave is running... \n\r" );
      _DBG( menu1 );
      _DBG( menu3 );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == 'x' || buffer == 'X' )
         {
            return;
         }
         else if( buffer == 'c' || buffer == 'C' )
         {
            break;
         }
      }

      // Receive --------------------------------------------------------

      _DBG( "Press '1' to Start \n\r" );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == '1' )
         {
            break;
         }
      }
      _DBG( "Receiving... \n\r" );

      // Initialize buffer
      Buffer_Init( Slave_Buf, 0 );
      transferSCfg.rx_data = Slave_Buf;
      transferSCfg.rx_length = 5;//3;
      HAL_I2C_Slave_Receive( ( I2Cn_Type* )I2C1, &transferSCfg, I2Cn_TRANSFER_POLLING );
      NOP();
      _DBG( "Received \n\r" );
      for( i = 0; i < BUFFER_SIZE; i++ )
      {
         _DBG( " 0x" );
         _DBH( Slave_Buf[i] );
      }
      _DBG( "\n\r" );

      // Receive and transmit --------------------------------------------------------

      _DBG( "Start Receive, wait for repeat start and transmit... \n\r" );

      // Initialize buffer
      Buffer_Init( Slave_Buf, 1 );
      transferSCfg.rx_data = S_buff;
      transferSCfg.rx_length = 2;
      Slave_Buf[0] = 0x8A; //data
      Slave_Buf[1] = 0xAA; //data
      Slave_Buf[2] = 0xA5; //data
      transferSCfg.tx_data = Slave_Buf;
      transferSCfg.tx_length = 3;
      HAL_I2C_SlaveTransferData( ( I2Cn_Type* )I2C1, &transferSCfg, I2Cn_TRANSFER_POLLING );
      NOP();
      _DBG( "Received \n\r" );
      for( i = 0; i < 2; i++ )
      {
         _DBG( " 0x" );
         _DBH( S_buff[i] );
      }
      _DBG( "\n\r" );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2C slave application using interrupt
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void App_I2C_Interrupt_Slave( void )
{
   I2Cn_S_SETUP_Type    transferSCfg;
   uint8_t              buffer = 0xFF;
   uint8_t              Slave_Buf[BUFFER_SIZE], S_buff[2];

   while( 1 )
   {
      _DBG( "I2Cn Interrupt Slave is running... \n\r" );
      _DBG( menu1 );
      _DBG( menu3 );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == 'x' || buffer == 'X' )
         {
            return;
         }
         else if( buffer == 'c' || buffer == 'C' )
         {
            break;
         }
      }

      // Receive --------------------------------------------------------

      _DBG( "Press '1' to Start \n\r" );
      while( 1 )
      {
         buffer = _DG();
         if( buffer == '1' )
         {
            break;
         }
      }
      _DBG( "Receiving... \n\r" );

      // Initialize buffer
      Buffer_Init( Slave_Buf, 0 );
      complete = FALSE;
      transferSCfg.rx_data = Slave_Buf;
      transferSCfg.rx_length = 5;//3;
      HAL_I2C_Slave_Receive( ( I2Cn_Type* )I2C1, &transferSCfg, I2Cn_TRANSFER_INTERRUPT );
      while( !complete );
      NOP();

      // Receive and transmit --------------------------------------------------------

      _DBG( "Start Receive, wait for repeat start and transmit... \n\r" );

      // Initialize buffer
      Buffer_Init( Slave_Buf, 0 );
      complete = FALSE;
      transferSCfg.rx_data = S_buff;
      transferSCfg.rx_length = 2;
      Slave_Buf[0] = 0x6A; //data
      Slave_Buf[1] = 0x8A; //data
      Slave_Buf[2] = 0xA5; //data
      transferSCfg.tx_data = Slave_Buf;
      transferSCfg.tx_length = 3;
      HAL_I2C_SlaveTransferData( ( I2Cn_Type* )I2C1, &transferSCfg, I2Cn_TRANSFER_INTERRUPT );
      while( !complete );
      NOP();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2C1_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void I2C1_IRQHandler_IT( void )
{
   if( isMasterMode )   // Master Mode
   {
      HAL_I2C_Master_IRQHandler_IT( ( I2Cn_Type* )I2C1 );
      if( HAL_I2C_Master_GetState( ( I2Cn_Type* )I2C1 ) )
      {
         complete = TRUE;
      }
   }
   else  // Slave Mode
   {
      HAL_I2C_Slave_IRQHandler_IT( ( I2Cn_Type* )I2C1 );
      if( HAL_I2C_Slave_GetState( ( I2Cn_Type* )I2C1 ) )
      {
         complete = TRUE;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2C1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void I2C1_Configure( void )
{
   // I2C1 configuration
   {
      // i2c SCL/SDA pin configuration
      {
         // print
         _DBG( "PB6 was configured as a SCL1. \n\r" );
         _DBG( "PB7 was configured as a SDA1. \n\r\n\r" );

         // configure PB6 as a SCL1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 6, ALTERN_FUNC );    // PB3            0: T41OUTA     1: T41INP      2: LPTXD       3: SCK0        4: SCL1        5: ----        6: ADTRG       7: SEG9
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 6, AFSRx_AF4 );    // PB6            0: T42OUTA     1: T42INP      2: ----        3: SCK0        4: SCL1        5: ----        6: ----        7: SEG6

         // configure PB7 as a SDA1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 7, ALTERN_FUNC );    // PB4            0: T41OUTB     1: EC41        2: LPRXD       3: ----        4: SDA1        5: ----        6: ADTRG       7: SEG8
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 7, AFSRx_AF4 );    // PB7            0: T42OUTB     1: EC42        2: RTCOUT      3: MISO0       4: SDA1        5: LPDE        6: ADTRG       7: SEG5
      }

      // i2c configuration
      {
         // Initialize Slave I2C peripheral
         HAL_I2C_Init( ( I2Cn_Type* )I2C1, 100000 );

         // Set Own slave address for I2C device
         HAL_I2C_Slave_SetAddress1( ( I2Cn_Type* )I2C1, I2CDEV_S_ADDR, DISABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( I2C1_IRQn, 3 );
      HAL_INT_EInt_MaskDisable( MSK_I2C1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2Cn_MasterSlaveRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void I2Cn_MasterSlaveRun( void )
{
   uint8_t     buffer = 0xFF;

   /* Infinite loop */
   while( 1 )
   {
      buffer = _DG();

      switch( buffer )
      {
         case '1':   // Master Polling Mode
            App_I2C_Polling_Master();
            DEBUG_MenuPrint();
            break;
         case '2':   // Master Interrupt Mode
            isMasterMode = TRUE;
            App_I2C_Interrupt_Master();
            DEBUG_MenuPrint();
            break;
         case '3':   // Slave Polling Mode
            App_I2C_Polling_Slave();
            DEBUG_MenuPrint();
            break;
         case '4':   // Slave Interrupt Mode
            isMasterMode = FALSE;
            App_I2C_Interrupt_Slave();
            DEBUG_MenuPrint();
            break;
         default:
            break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* I2C1_Configure */
   I2C1_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* I2Cn_MasterSlaveRun */
   I2Cn_MasterSlaveRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

