/***************************************************************************//**
* @file     lcd.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

//******************************************************************************
// Variable
//******************************************************************************

static uint8_t*      lcdfbuf;

static const char    LEDSEG[17] =
{
   // 0    1    2    3    4    5    6    7    8    9    A    b    c    d    E    F    ' '
   // 0x0f,0x00,0x0d,0x09,0x02,0x0b,0x0f,0x03,0x0f,0x0b,0x07,0x0e,0x0f,0x0c,0x0f,0x07,0x00
   0x0f, 0x00, 0x0b, 0x09, 0x04, 0x0d, 0x0f, 0x0c, 0x0f, 0x0d, 0x0e, 0x07, 0x0f, 0x03, 0x0f, 0x0e, 0x00
};

static const char    LEDSEGn[17] =
{
   // 0    1    2    3    4    5    6    7    8    9    A    b    c    d    E    F    ' '
   // 0x05,0x05,0x03,0x07,0x07,0x06,0x06,0x05,0x07,0x07,0x07,0x06,0x00,0x07,0x02,0x02,0x00
   0x0a, 0x0a, 0x0c, 0x0e, 0x0e, 0x06, 0x06, 0x0a, 0x0e, 0x0e, 0x0e, 0x06, 0x00, 0x0e, 0x0e, 0x0e, 0x00
};

static char          ring_state;

//******************************************************************************
// Function
//******************************************************************************

static void lcd_setting( void )
{
   char              i;
   LCD_CFG_Type      LCD_Config;

   // select peripheral clock: WDTRC
   HAL_SCU_Peripheral_ClockSelection( PPCLKSR_LCDCLK, LCDCLK_WDTRC );

   // init lcd
   LCD_Config.Bias = IRSEL_RLCD2;
   LCD_Config.Duty = DBS_4D3B;
   LCD_Config.Clk = LCLK_128Hz;
   LCD_Config.AutoBiasEn = LCDABC_Off;
   LCD_Config.BiasTime = BMSEL_BMA4Clk;
   LCD_Config.Contrast = LCTEN_Disable;
   LCD_Config.ContrastStep = VLCD_Step0;
   HAL_LCD_Init( &LCD_Config );

   // start address of Display Data RAM
   lcdfbuf = ( uint8_t* )( 0x40005010 );     // &LCD->DR0

   // init Display Data RAM
   for( i = 0; i < LCDBufSize; i++ )
   {
      lcdfbuf[i] = 0x00;
   }

   // lcd bias voltage pin configuration
   {
      // configure PD3 as a VLC0
      _DBG( "PD3 was configured as a VLC0. \n\r" );
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 3, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 3, 7 );     // PD3            0: ----        1: ----        2: TXD0        3: ----        4: SC1IN       5: SCL0        6: CP1P4       7: VLC0
   }

   // lcd signal pin configuration
   {
      // print
      _DBG( "PC0 ~ PC2 was configured as a COM0 ~ COM2. \n\r" );
      _DBG( "PD7 was configured as a COM3. \n\r" );
      _DBG( "PB0 ~ PB12 was configured as a SEG12 ~ SEG0. \n\r" );
      _DBG( "PA6 ~ PA9 was configured as a SEG16 ~ SEG13. \n\r" );
      _DBG( "PA1 ~ PA3 was configured as a SEG19 ~ SEG17. \n\r" );
      _DBG( "PF0 ~ PF3 was configured as a SEG23 ~ SEG20. \n\r" );
      _DBG( "PC7 was configured as a SEG24. \n\r" );
      _DBG( "PC8 ~ PC11 was configured as a COM4/SEG25 ~ COM7/SEG28 (except for PC10 ~ PC11). \n\r" );
      _DBG( "PD0 ~ PD2 was configured as a SEG29 ~ SEG31. \n\r" );
      _DBG( "PD6 was configured as a SEG32. \n\r" );

      // configure PC0 ~ PC2 as a COM0 ~ COM2
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 2, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 2, 7 );        // PC2            0: ----        1: ----        2: RXD0        3: ----        4: SC0CLK      5: ----        6: ----        7: COM2
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 1, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 1, 7 );        // PC1   CLKO     0: CLKO        1: ----        2: TXD0        3: ----        4: SC0PWR      5: ----        6: ----        7: COM1
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 0, 7 );        // PC0   CLKO     0: CLKO        1: ----        2: ----        3: ----        4: SC0IN       5: ----        6: ----        7: COM0

      // configure PD7 as a COM3
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 7, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 7, 7 );        // PD7            0: ----        1: ----        2: ----        3: SS0         4: SDA0        5: ----        6: ----        7: COM3

      // configure PB0 ~ PB12 as a SEG12 ~ SEG0
      PB->MOD     =  0
                     | ( 2 << PB_PB_MOD_MODE12_Pos )        // PB12           0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE11_Pos )        // PB11           0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE10_Pos )        // PB10           0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE9_Pos )         // PB9            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE8_Pos )         // PB8            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE7_Pos )         // PB7            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE6_Pos )         // PB6            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE5_Pos )         // PB5            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE4_Pos )         // PB4            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE3_Pos )         // PB3            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE2_Pos )         // PB2            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE1_Pos )         // PB1            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PB_PB_MOD_MODE0_Pos )         // PB0            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     ;
      PB->AFSR2   =  0
                     | ( 7 << PB_PB_AFSR2_AFSR12_Pos )      // PB12           0: T42INP      1: ----        2: ----        3: ----        4: ----        5: ----        6: ----        7: SEG0
                     | ( 7 << PB_PB_AFSR2_AFSR11_Pos )      // PB11           0: T43INP      1: ----        2: ----        3: ----        4: ----        5: ----        6: ----        7: SEG1
                     | ( 7 << PB_PB_AFSR2_AFSR10_Pos )      // PB10           0: T43OUTB     1: EC43        2: ----        3: ----        4: ----        5: ----        6: ----        7: SEG2
                     | ( 7 << PB_PB_AFSR2_AFSR9_Pos )       // PB9            0: T43OUTA     1: T43INP      2: ----        3: ----        4: ----        5: ----        6: ----        7: SEG3
                     | ( 7 << PB_PB_AFSR2_AFSR8_Pos )       // PB8            0: ----        1: ----        2: ----        3: MOSI0       4: ----        5: ----        6: ----        7: SEG4
                     ;
      PB->AFSR1   =  0
                     | ( 7 << PB_PB_AFSR1_AFSR7_Pos )       // PB7            0: T42OUTB     1: EC42        2: RTCOUT      3: MISO0       4: SDA1        5: LPDE        6: ADTRG       7: SEG5
                     | ( 7 << PB_PB_AFSR1_AFSR6_Pos )       // PB6            0: T42OUTA     1: T42INP      2: ----        3: SCK0        4: SCL1        5: ----        6: ----        7: SEG6
                     | ( 7 << PB_PB_AFSR1_AFSR5_Pos )       // PB5            0: ----        1: ----        2: ----        3: SS0         4: ----        5: LPDE        6: ----        7: SEG7
                     | ( 7 << PB_PB_AFSR1_AFSR4_Pos )       // PB4            0: T41OUTB     1: EC41        2: LPRXD       3: ----        4: SDA1        5: ----        6: ADTRG       7: SEG8
                     | ( 7 << PB_PB_AFSR1_AFSR3_Pos )       // PB3            0: T41OUTA     1: T41INP      2: LPTXD       3: SCK0        4: SCL1        5: ----        6: ADTRG       7: SEG9
                     | ( 7 << PB_PB_AFSR1_AFSR2_Pos )       // PB2            0: T50OUT      1: ----        2: ----        3: ----        4: ----        5: ----        6: ----        7: SEG10
                     | ( 7 << PB_PB_AFSR1_AFSR1_Pos )       // PB1            0: ----        1: ----        2: RXD1        3: ----        4: AN11        5: LPDE        6: ----        7: SEG11
                     | ( 7 << PB_PB_AFSR1_AFSR0_Pos )       // PB0            0: ----        1: ----        2: TXD1        3: ----        4: AN10        5: ----        6: ----        7: SEG12
                     ;

      // configure PA6 ~ PA9 as a SEG16 ~ SEG13
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 9, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 9, 7 );        // PA9            0: ----        1: ----        2: LPRXD       3: ----        4: AN9         5: ----        6: ----        7: SEG13
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 8, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 8, 7 );        // PA8            0: ----        1: ----        2: LPTXD       3: ----        4: AN8         5: ----        6: ----        7: SEG14
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 7, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 7, 7 );        // PA7            0: T43OUTB     1: EC43        2: ----        3: MOSI1       4: AN7         5: CP1P1       6: CP1OUT      7: SEG15
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 6, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 6, 7 );        // PA6            0: T43OUTA     1: T43INP      2: ----        3: MISO1       4: AN6         5: ----        6: CP0OUT      7: SEG16

      // configure PA1 ~ PA3 as a SEG19 ~ SEG17
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 3, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 3, 7 );        // PA3            0: T41OUTB     1: EC41        2: RXD10       3: MISO10      4: AN3         5: CP1P0       6: ADTRG       7: SEG17
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 2, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 2, 7 );        // PA2            0: T41OUTA     1: T41INP      2: TXD10       3: MOSI10      4: AN2         5: CP1N0       6: CP1OUT      7: SEG18
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 1, 7 );        // PA1            0: T40OUTB     1: EC40        2: T42INP      3: SS10        4: AN1         5: CP0P0       6: ----        7: SEG19

      // configure PF0 ~ PF3 as a SEG23 ~ SEG20
      PF->MOD     =  0
                     | ( 2 << PF_PF_MOD_MODE3_Pos )         // PF3            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PF_PF_MOD_MODE2_Pos )         // PF2            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PF_PF_MOD_MODE1_Pos )         // PF1            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     | ( 2 << PF_PF_MOD_MODE0_Pos )         // PF0            0: Input Mode     1: Output Mode    2: Alternative Function Mode
                     ;
      PF->AFSR1   =  0
                     | ( 7 << PF_PF_AFSR1_AFSR3_Pos )       // PF3            0: ----        1: ----        2: ----        3: MOSI0       4: AN15        5: ----        6: ----        7: SEG20
                     | ( 7 << PF_PF_AFSR1_AFSR2_Pos )       // PF2            0: ----        1: T50INP      2: ----        3: MISO0       4: AN14        5: ----        6: ----        7: SEG21
                     | ( 7 << PF_PF_AFSR1_AFSR1_Pos )       // PF1            0: T50OUT      1: ----        2: ----        3: ----        4: AN13        5: ----        6: ----        7: SEG22
                     | ( 7 << PF_PF_AFSR1_AFSR0_Pos )       // PF0            0: ----        1: EC50        2: ----        3: ----        4: AN12        5: ----        6: ----        7: SEG23
                     ;

      // configure PC7 as a SEG24
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 7, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 7, 7 );        // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24

      // configure PC8 ~ PC11 as a COM4/SEG25 ~ COM7/SEG28 (except for PC10 ~ PC11)
      // HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 11, ALTERN_FUNC );
      // HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 11, 7 );       // PC11           0: ----        1: ----        2: RXD1        3: ----        4: ----        5: LPDE        6: ----        7: COM7/SEG28
      // HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 10, ALTERN_FUNC );
      // HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 10, 7 );       // PC10           0: ----        1: ----        2: TXD1        3: ----        4: SC0IN       5: ----        6: ----        7: COM6/SEG27
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 9, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 9, 7 );        // PC9            0: ----        1: ----        2: LPRXD       3: SS10        4: SC0PWR      5: ----        6: ----        7: COM5/SEG26
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 8, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 8, 7 );        // PC8            0: ----        1: ----        2: LPTXD       3: SCK10       4: SC0CLK      5: ----        6: ----        7: COM4/SEG25

      // configure PD0 ~ PD2 as a SEG29 ~ SEG31
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, 7 );        // PD2            0: T43OUTB     1: EC43        2: EC50        3: MOSI1       4: SC1DATA     5: SC1TXD      6: CP1P3       7: SEG31
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 1, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 1, 7 );        // PD1            0: T43OUTA     1: T43INP      2: ----        3: MISO1       4: SC1RST      5: SC1RXD      6: CP1P2       7: SEG30
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 0, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 0, 7 );        // PD0            0: T40OUTB     1: EC40        2: ----        3: SCK1        4: SC1CLK      5: CP1N3       6: ----        7: SEG29

      // configure PD6 as a SEG32
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 6, ALTERN_FUNC );
      HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 6, 7 );        // PD6            0: ----        1: ----        2: ----        3: ----        4: SCL0        5: ----        6: ----        7: SEG32
   }
}

static void l_init( void )
{
   int      i;

   for( i = 5; i < LCDBufSize; i++ )
   {
      lcdfbuf[i] = 0x00;
   }
}

static void l_disp( unsigned int disp )
{
   char     tch[6];

   tch[0] = disp % 10;
   disp /= 10;
   if( disp == 0 )
   {
      tch[1] = tch[2] = tch[3] = 16;
   }
   else
   {
      tch[1] = disp % 10;
      disp /= 10;
      if( disp == 0 )
      {
         tch[2] = tch[3] = 16;
      }
      else
      {
         tch[2] = disp % 10;
         disp /= 10;
         if( disp == 0 )
         {
            tch[3] = 16;
         }
         else
         {
            tch[3] = disp % 10;
         }
      }
   }

   lcdfbuf[5] = LEDSEG[tch[3]];
   lcdfbuf[6] = LEDSEGn[tch[3]];
   lcdfbuf[7] = LEDSEG[tch[2]];
   lcdfbuf[8] = LEDSEGn[tch[2]];
   lcdfbuf[9] = LEDSEG[tch[1]];
   lcdfbuf[10] = LEDSEGn[tch[1]];
   lcdfbuf[11] = LEDSEG[tch[0]];
   lcdfbuf[12] = LEDSEGn[tch[0]];
}

static void l_ring( char dir )
{
   char     tch[8];

   if( dir )
   {
      ring_state++;
      if( ring_state > 11 )
      {
         ring_state = 0;
      }
   }
   else
   {
      ring_state--;
      if( ring_state == 0xff )
      {
         ring_state = 11;
      }
   }

   switch( ring_state )
   {
      case  0:
         tch[0] = 0x08;
         tch[1] = 0   ;
         tch[2] = 0x08;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case  1:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0x08;
         tch[3] = 0   ;
         tch[4] = 0x08;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case  2:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0x08;
         tch[5] = 0   ;
         tch[6] = 0x08;
         tch[7] = 0   ;
         break;
      case  3:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0x08;
         tch[7] = 0x08;
         break;
      case  4:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0x0a;
         break;
      case  5:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0x01;
         tch[7] = 0x02;
         break;
      case  6:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0x01;
         tch[5] = 0   ;
         tch[6] = 0x01;
         tch[7] = 0   ;
         break;
      case  7:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0x01;
         tch[3] = 0   ;
         tch[4] = 0x01;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case  8:
         tch[0] = 0x01;
         tch[1] = 0   ;
         tch[2] = 0x01;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case  9:
         tch[0] = 0x03;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case 10:
         tch[0] = 0x06;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case 11:
         tch[0] = 0x0c;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      default:
         break;
   }

   DI();
   lcdfbuf[5] = tch[0];
   lcdfbuf[6] = tch[1];
   lcdfbuf[7] = tch[2];
   lcdfbuf[8] = tch[3];
   lcdfbuf[9] = tch[4];
   lcdfbuf[10] = tch[5];
   lcdfbuf[11] = tch[6];
   lcdfbuf[12] = tch[7];
   lcdfbuf[13] = 0;
   EI();
}

static void l_bar( void )
{
   DI();
   lcdfbuf[5] = 0;
   lcdfbuf[6] = 0x04;
   lcdfbuf[7] = 0;
   lcdfbuf[8] = 0x04;
   lcdfbuf[9] = 0;
   lcdfbuf[10] = 0x04;
   lcdfbuf[11] = 0;
   lcdfbuf[12] = 0x04;
   lcdfbuf[13] = 0;
   EI();
}

static void lcd_test( void )
{
   uint8_t        ch;
   char           cmd;
   unsigned int   count = 0;

   cmd = '2';

   LCDON();

   for( ;; )
   {
      if( _DG_( &ch ) )
      {
         cmd = ch;
         _DBC( ch );
         _DBG( "\n\r" );
      }

      switch( cmd )
      {
         case '1':
            DI();
            l_init();
            EI();
            cmd = 'x';
            break;
         case '2':
            l_disp( count++ );
            msec = 500;
            while( msec );
            break;
         case '3':
            l_ring( 0 );
            msec = 80;
            while( msec );
            count = 0;
            break;
         case '4':
            l_ring( 1 );
            msec = 120;
            while( msec );
            count = 0;
            break;
         case '5':
            l_bar();
            msec = 120;
            while( msec );
            count = 0;
            break;
         case '6':
            msec = 600;
            while( msec );
            count = 0;
            break;
         case '7':
            msec = 800;
            while( msec );
            count = 0;
            break;
         case 'q':
         case 'Q':
            return;
         default:
            break;
      }
   }
}

void t_lcd( void )
{
   char     cmd;

   DI();
   lcd_setting();
   EI();

   while( 1 )
   {
      _DBG( "\n\rLCD test Program\n\r" );
      _DBG( "1. LCD OFF\n\r" );
      _DBG( "2. LCD ON/Count Number\n\r" );
      _DBG( "3. LCD Ring Rotation Left\n\r" );
      _DBG( "4. LCD Ring Rotation Right\n\r" );
      _DBG( "5. LCD Center Bar\n\r" );
      _DBG( "q. return to main.\n\r" );

      _DBG( ">" );
      lcd_test();

      getstring();
      if( InFlag )
      {
         _DBG( "\n" );
         cmd = InData[0];

         switch( cmd )
         {
            case '1':
               LCDON();
               break;
            case '2':
               LCDOFF();
               break;
            case '3':
               lcd_test();
               break;
            case 'q':
            case 'Q':
               _DBG( "Bye~\n\r" );
               return;
         }
      }
      else
      {
         _DBG( "\n" );
         InFlag = 0;
         InCount = 0;
      }
   }
}

