/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void IntTransmit( void );
void IntReceive( void );

uint32_t rbSend( uint8_t* txbuf, uint8_t buflen );
uint32_t rbReceive( uint8_t* rxbuf, uint8_t buflen );
void LPUART_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void LPUART_Configure( void );
void LPUART_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// ring buffer size
#define RING_BUF_SIZE      32

//******************************************************************************
// Type
//******************************************************************************

typedef struct
{
   uint32_t    tx_head;                /**<  Tx ring buffer head index */
   uint32_t    tx_tail;                /**<  Tx ring buffer tail index */
   uint32_t    rx_head;                /**<  Rx ring buffer head index */
   uint32_t    rx_tail;                /**<  Rx ring buffer tail index */
   uint8_t     tx[RING_BUF_SIZE];      /**<  Tx data ring buffer */
   uint8_t     rx[RING_BUF_SIZE];      /**<  Rx data ring buffer */
} RING_BUF;

//******************************************************************************
// Macro
//******************************************************************************

// Buf mask
#define __BUF_MASK                        ( RING_BUF_SIZE - 1 )

// Check buf is full or not
#define __BUF_IS_FULL( head, tail )       ( (tail & __BUF_MASK) == ((head+1) & __BUF_MASK) )

// Check buf will be full in next receiving or not
#define __BUF_WILL_FULL( head, tail )     ( (tail & __BUF_MASK) == ((head+2) & __BUF_MASK) )

// Check buf is empty
#define __BUF_IS_EMPTY( head, tail )      ( (head & __BUF_MASK) == (tail & __BUF_MASK) )

// Reset buf
#define __BUF_INCR( bufidx )              ( bufidx = (bufidx+1) & __BUF_MASK )

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "LPUART_Interrupt Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: LPUART \n\r"
                        "Counterpart:       A external device like a PC \n\r"
                        "Tx Data:           Any character \n\r"
                        "UART TXD Pin:      PA8(LPTXD) \n\r"
                        "UART RXD Pin:      PA9(LPRXD) \n\r"
                        "************************************************\n\r\n\r";

// ring buffer
RING_BUF                rb;

// Current Tx Interrupt enable state
volatile FlagStatus     TxIntStat;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         UART transmit function (ring buffer used)
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void IntTransmit( void )
{
   // Disable THRE interrupt
   HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_TXCIEN, DISABLE );




   while( !__BUF_IS_EMPTY( rb.tx_head, rb.tx_tail ) )
   {
      /* Move a piece of data into the transmit FIFO */
      if( HAL_LPUART_Transmit( ( uint8_t* )&rb.tx[rb.tx_tail], 1, NONE_BLOCKING ) )
      {
         /* Update transmit ring FIFO tail pointer */
         __BUF_INCR( rb.tx_tail );
      }
      else
      {
         break;
      }
   }

   /* If there is no more data to send, disable the transmit
      interrupt - else enable it or keep it enabled */
   if( __BUF_IS_EMPTY( rb.tx_head, rb.tx_tail ) )
   {
      HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_TXCIEN, DISABLE );
      // Reset Tx Interrupt state
      TxIntStat = RESET;
   }
   else
   {
      // Set Tx Interrupt state
      TxIntStat = SET;
      HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_TXCIEN, ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART receive function (ring buffer used)
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void IntReceive( void )
{
   uint8_t     tmpc;
   uint32_t    rLen;

   while( 1 )
   {
      // Call UART receive function in UART driver
      rLen = HAL_LPUART_Receive( &tmpc, 1, NONE_BLOCKING );

      // If data received
      if( rLen )
      {
         /* Check if buffer is more space
          * If no more space, remaining character will be trimmed out
          */
         if( !__BUF_IS_FULL( rb.rx_head, rb.rx_tail ) )
         {
            rb.rx[rb.rx_head] = tmpc;
            __BUF_INCR( rb.rx_head );
         }
      }
      // no more data
      else
      {
         break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         LPUART transmit function for interrupt mode (using ring buffers)
 * @param[in]     txbuf
 *                   Pointer to Transmit buffer
 * @param[in]     buflen
 *                   Length of Transmit buffer
 * @return        Number of bytes actually sent to the ring buffer
 *//*-------------------------------------------------------------------------*/
uint32_t rbSend( uint8_t* txbuf, uint8_t buflen )
{
   uint8_t*    data = ( uint8_t* ) txbuf;
   uint32_t    bytes = 0;

   /* Temporarily lock out UART transmit interrupts during this
      read so the UART transmit interrupt won't cause problems
      with the index values */
   HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_TXCIEN, DISABLE );

   /* Loop until transmit run buffer is full or until n_bytes expires */
   while( ( buflen > 0 ) && ( !__BUF_IS_FULL( rb.tx_head, rb.tx_tail ) ) )
   {
      /* Write data from buffer into ring buffer */
      rb.tx[rb.tx_head] = *data;
      data++;

      /* Increment head pointer */
      __BUF_INCR( rb.tx_head );

      /* Increment data count and decrement buffer size count */
      bytes++;
      buflen--;
   }

   /*
    * Check if current Tx interrupt enable is reset,
    * that means the Tx interrupt must be re-enabled
    * due to call IntTransmit() function to trigger
    * this interrupt type
    */
   if( TxIntStat == RESET )
   {
      IntTransmit();
   }
   /*
    * Otherwise, re-enables Tx Interrupt
    */
   else
   {
      HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_TXCIEN, ENABLE );
   }

   return bytes;
}

/*-------------------------------------------------------------------------*//**
 * @brief         LPUART read function for interrupt mode (using ring buffers)
 * @param[out]    rxbuf
 *                   Pointer to Received buffer
 * @param[in]     buflen
 *                   Length of Received buffer
 * @return        Number of bytes actually read from the ring buffer
 *//*-------------------------------------------------------------------------*/
uint32_t rbReceive( uint8_t* rxbuf, uint8_t buflen )
{
   uint8_t*    data = ( uint8_t* ) rxbuf;
   uint32_t    bytes = 0;

   /* Temporarily lock out UART receive interrupts during this
      read so the UART receive interrupt won't cause problems
      with the index values */
   HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_RXCIEN, DISABLE );

   /* Loop until receive buffer ring is empty or until max_bytes expires */
   while( ( buflen > 0 ) && ( !( __BUF_IS_EMPTY( rb.rx_head, rb.rx_tail ) ) ) )
   {
      /* Read data from ring buffer into user buffer */
      *data = rb.rx[rb.rx_tail];
      data++;

      /* Update tail pointer */
      __BUF_INCR( rb.rx_tail );

      /* Increment data count and decrement buffer size count */
      bytes++;
      buflen--;
   }

   /* Re-enable UART interrupts */
   HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_RXCIEN, ENABLE );

   return bytes;
}

/*-------------------------------------------------------------------------*//**
 * @brief         LPUART_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void LPUART_IRQHandler_IT( void )
{
   uint32_t    intsrc;

   // get interrupt flag
   intsrc = HAL_LPUART_GetStatus();

   // if Receive Complete Interrupt
   if( ( intsrc & LPUART_IFSR_RXCIFLAG_Msk ) == LPUART_IFSR_RXCIFLAG_Msk )
   {
      IntReceive();
   }

   // if Transmit Complete Interrupt
   if( ( intsrc & LPUART_IFSR_TXCIFLAG_Msk ) == LPUART_IFSR_TXCIFLAG_Msk )
   {
      IntTransmit();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         LPUART_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void LPUART_Configure( void )
{
   LPUART_CFG_Type      LPUART_Config;

   // init flag
   TxIntStat = RESET;

   // Reset ring buf head and tail idx
   rb.rx_head = 0;
   rb.rx_tail = 0;
   rb.tx_head = 0;
   rb.tx_tail = 0;

   // LPUART configuration
   {
      // uart TXD/RXD pin configuration
      {
         // print
         _DBG( "PA8 was configured as a LPTXD. \n\r" );
         _DBG( "PA9 was configured as a LPRXD. \n\r\n\r" );

         // configure PA8 as a LPTXD                              // PA8            0: ----        1: ----        2: LPTXD       3: ----        4: AN8         5: ----        6: ----        7: SEG14
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 8, ALTERN_FUNC );      // PB3            0: T41OUTA     1: T41INP      2: LPTXD       3: SCK0        4: SCL1        5: ----        6: ADTRG       7: SEG9
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 8, AFSRx_AF2 );   // PC8            0: ----        1: ----        2: LPTXD       3: SCK10       4: SC0CLK      5: ----        6: ----        7: COM4/SEG25

         // configure PA9 as a LPRXD                              // PA9            0: ----        1: ----        2: LPRXD       3: ----        4: AN9         5: ----        6: ----        7: SEG13
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 9, ALTERN_FUNC );      // PB4            0: T41OUTB     1: EC41        2: LPRXD       3: ----        4: SDA1        5: ----        6: ADTRG       7: SEG8
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 9, AFSRx_AF2 );   // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24
      }

      // uart configuration
      {
         // get default configuration for uart: 38400-8-N-1
         HAL_LPUART_ConfigStructInit( &LPUART_Config );

         // select peripheral clock: PCLK
         HAL_SCU_Peripheral_ClockSelection( PPCLKSR_LPUTCLK, LPUTCLK_PCLK );
         LPUART_Config.BaseClock = SystemPeriClock;

         // init LPUART
         HAL_LPUART_Init( &LPUART_Config );

         // enable interrupt
         HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_RXCIEN, ENABLE );
         HAL_LPUART_ConfigInterrupt( LPUART_INTCFG_TXCIEN, ENABLE );

         // enable LPUART
         HAL_LPUART_Enable( ENABLE );
      }

      // unmask interrupt
      NVIC_SetPriority( LPUART_IRQn, 3 );
      NVIC_EnableIRQ( LPUART_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_LPUART );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         LPUART_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void LPUART_InterruptRun( void )
{
   uint8_t     buf[100];   // tx/rx buffer
   uint32_t    len;

   /* Infinite loop */
   while( 1 )
   {
      // receive data from PC
      len = rbReceive( buf, sizeof( buf ) );

      // send the received data to PC
      if( len > 0 )
      {
         rbSend( buf, len );
         if( buf[len - 1] == '\r' )
         {
            buf[0] = '\n';
            rbSend( buf, 1 );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* LPUART_Configure */
   LPUART_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* LPUART_InterruptRun */
   LPUART_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

