/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void EINT0_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void EINT_Configure( void );
void PMU_SRAMRetentionPowerRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "PMU_SRAMRetentionPower Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:                           PMU \n\r"
                        "GPIO Output Pin:                             PB0 ~ PB3 \n\r"
                        "GPIO Input Pin for Deep Sleep Mode Entrance: PA1 \n\r"
                        "EINT Input Pin for Deep Sleep Mode Exit:     PA0(EINT0) \n\r"
                        "************************************************\n\r\n\r";

uint32_t    LoopCounter = 0;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         EINT0_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void EINT0_IRQHandler_IT( void )
{
   uint32_t    status;

   // get interrupt flag
   status = HAL_INT_EIntPA_GetIntStatus();

   // if interrupt was issued by PA0
   if( status & _BIT( 0 ) )
   {
      // clear interrupt flag
      HAL_INT_EIntPA_ClearIntStatus( _BIT( 0 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PB0 ~ PB3 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );

      // configure PB1 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );

      // configure PB2 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 2, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );

      // configure PB3 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 3, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
   }

   // gpio input pin configuration
   {
      // print
      _DBG( "PA1 was configured as a GPIO Input Mode. \n\r\n\r" );

      // configure PA1 as a Input Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, INPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 1, PUPDx_EnablePU );
      HAL_GPIO_SetDebouncePin( ( Pn_Type* )PA, 1, DBCLK_HCLK1024 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         EINT_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void EINT_Configure( void )
{
   // eint input pin configuration
   {
      // PA0 configuration
      {
         // print
         _DBG( "PA0 was configured as a EINT Input Mode. \n\r" );
         _DBG( "PA0 EINT was configured as a Falling Edge Trigger. \n\r" );
         _DBG( "EINT0 was configured for a PA0 EINT. \n\r\n\r" );

         // configure PA0 as a Input Mode
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 0, INPUT );
         HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 0, PUPDx_EnablePU );
         // HAL_GPIO_SetDebouncePin( ( Pn_Type* )PA, 0, DBCLK_HCLK1024 );

         // configure PA0 as a Falling Edge Trigger
         HAL_INT_EIntPx_SetReg( PORTA, 0, ITRIGx_Edge, INTCTLx_FallingEdge );

         // configure EINT0 for PA0 EINT
         HAL_INT_EIntCfg( EINT0, CONFx_PA, 0 );
      }

      // unmask Interrupt
      NVIC_SetPriority( EINT0_IRQn, 3 );
      NVIC_EnableIRQ( EINT0_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_EINT0 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         PMU_SRAMRetentionPowerRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void PMU_SRAMRetentionPowerRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
      // print loop counter
      _DBD32( LoopCounter );
      _DBG( "  Press PA1 to enter Deep Sleep Mode. \n\r" );

      // check user's order
      if( ( HAL_GPIO_ReadPin( ( Pn_Type* )PA ) & ( 1uL <<  1 ) ) == 0 )
      {
         // go to Deep Sleep Mode
         _DBG( "\n\r" );
         _DBG( "Press PA0 to exit Deep Sleep Mode. \n\r\n\r" );
         HAL_SCU_ClockMonitoring_Disable();
         HAL_PWR_EnterPowerDownMode();
         HAL_SCU_ClockMonitoring( MACTS_SysClkChg, MONCS_MCLK );
      }

      // output high
      HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );
      HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 1 ) );
      HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 2 ) );
      HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 3 ) );
      delay();

      // output low
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
      delay();

      // increase LoopCounter by 1
      LoopCounter++;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* EINT_Configure */
   EINT_Configure();

   // disable LVR (Low Voltage Reset by H/W, Option)
   SCULV_DisLVR();

   // enable sram retention power
   HAL_PMU_Enable_SRAMRetentionPower();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* PMU_SRAMRetentionPowerRun */
   PMU_SRAMRetentionPowerRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

