/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void SCIn_Configure( void );
void SCn_SCI_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Type
//******************************************************************************

//==============================================================================
// Enumeration
//==============================================================================

// Card State Definition
typedef enum
{
   CARD_NONE = 0,
   CARD_START,
   CARD_ACTIVATION,
   CARD_WAIT_ATR,
   CARD_DEACTIVATION,
   CARD_INTERFACE
} CARD_STATE_Type;

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "SCn_SCI_Polling Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: SCI - 5434-8-E-1 \n\r"
                        "Target Peripheral: SC0 \n\r"
                        "Counterpart:       SC1: Only Reflect \n\r"
                        "Tx Data:           Any character \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         SCIn_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SCIn_Configure( void )
{
   SCn_CFG_Type   SCn_Config;

   // SCI0 ~ SCI1 configuration
   {
      // sci IN/PWR/CLK/RST/DATA pin configuration
      {
         // print
         _DBG( "PC[4:0] was configured as a SCI mode pins of SC0. \n\r" );
         _DBG( "PD2: SC1DATA, Connect to SC0DATA pin. \n\r" );
         _DBG( "PD4: Output To active the SC0 block. Connect to SC0IN pin. \n\r\n\r" );

         // configure PC0 as a SC0IN
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, ALTERN_FUNC );      // PC0            0: CLKO        1: ----        2: ----        3: ----        4: SC0IN       5: ----        6: ----        7: COM0
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 0, AFSRx_AF4 );   // PC10           0: ----        1: ----        2: TXD1        3: ----        4: SC0IN       5: ----        6: ----        7: COM6/SEG27
         HAL_GPIO_SetDebouncePin( ( Pn_Type* )PC, 0, DBCLK_HCLK64 );

         // configure PC1 as a SC0PWR
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 1, ALTERN_FUNC );      // PC1            0: CLKO        1: ----        2: TXD0        3: ----        4: SC0PWR      5: ----        6: ----        7: COM1
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 1, AFSRx_AF4 );   // PC9            0: ----        1: ----        2: LPRXD       3: SS 10       4: SC0PWR      5: ----        6: ----        7: COM5/SEG26

         // configure PC2 as a SC0CLK
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 2, ALTERN_FUNC );      // PC2            0: ----        1: ----        2: RXD0        3: ----        4: SC0CLK      5: ----        6: ----        7: COM2
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 2, AFSRx_AF4 );   // PC8            0: ----        1: ----        2: LPTXD       3: SCK10       4: SC0CLK      5: ----        6: ----        7: COM4/SEG25

         // configure PC3 as a SC0RST
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 3, ALTERN_FUNC );      // PC3            0: ----        1: ----        2: ----        3: MISO1       4: SC0RST      5: SC0RXD      6: CP0OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 3, AFSRx_AF4 );   // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24

         // configure PC4 as a SC0DATA
         PC->PC_TYP_b.TYP4 = Pn_TYP_TYP4_OpenDrain;
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 4, ALTERN_FUNC );      // PC4            0: ----        1: ----        2: ----        3: MOSI1       4: SC0DATA     5: SC0TXD      6: CP1OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 4, AFSRx_AF4 );   // PC6   SWCLK    0: SWCLK       1: ----        2: TXD10       3: MOSI10      4: SC0DATA     5: SC0TXD      6: ----        7: ----

         // configure PD4 as a output
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, PUSH_PULL_OUTPUT );

         // configure PD2 as a SC1DATA
         PD->PD_TYP_b.TYP2 = Pn_TYP_TYP2_OpenDrain;
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );      // PD2            0: T43OUTB     1: EC43        2: EC50        3: MOSI1       4: SC1DATA     5: SC1TXD      6: CP1P3       7: SEG31
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, AFSRx_AF4 );
      }

      // sci configuration
      {
         // get default SCn_Config for SCI: 8-Y-1
         HAL_SC_ConfigStructInit( &SCn_Config, SCn_SCI_MODE, 5434 );

         // init SCn
         SCn_Config.Baud_rate_Compensation = 0;
         HAL_SC_Init( ( SCn_Type* )SC0, &SCn_Config );
         HAL_SC_Init( ( SCn_Type* )SC1, &SCn_Config );

         // enable SCn
         HAL_SC_Enable( ( SCn_Type* )SC0, ENABLE );

         HAL_SC_Enable( ( SCn_Type* )SC1, ENABLE );
         HAL_SC_AutoAct( ( SCn_Type* )SC1 );
         delay();
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         SCn_SCI_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SCn_SCI_PollingRun( void )
{
   uint32_t             len_pc, len_sc0, len_sc1, intsrc;
   uint8_t              buffer_pc[100], buffer_sc0[100], buffer_sc1[100];
   CARD_STATE_Type      card_state = CARD_NONE;

   /* Infinite loop */
   while( 1 )
   {
      switch( card_state )
      {
         case CARD_NONE:
            // to set high level at SC0IN Pin for test
            GPIO_SetBit4( PD );
            _DBG( "The PD[4] outputs high level to test SCI mode. \n\r" );
            delay();
            card_state = CARD_START;
            break;
         case CARD_START:
            if( HAL_SC_GetSCnINST( ( SCn_Type* )SC0 ) )
            {
               HAL_SC_AutoConvDet( ( SCn_Type* )SC0 );
               HAL_SC_AutoAct( ( SCn_Type* )SC0 );
               card_state = CARD_ACTIVATION;
            }
            break;
         case CARD_ACTIVATION:
            intsrc = HAL_SC_GetStatus( ( SCn_Type* )SC0 );
            intsrc &= SCn_IFSR_BITMASK;

            // if Reset Assertion Interrupt
            if( ( intsrc & SCn_IFSR_RSTA ) == SCn_IFSR_RSTA )
            {
               _DBG( "The value of SC0->IFSR is " );
               _DBH32( intsrc );
               _DBG( ". Reset assertion finish. \n\r" );
               HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_RSTA );
               card_state = CARD_WAIT_ATR;
               break;
            }

            // if Sequence Error Interrupt
            if( ( intsrc & SCn_IFSR_SER ) == SCn_IFSR_SER )
            {
               _DBG( "The value of SC0->IFSR is " );
               _DBH32( intsrc );
               _DBG( ". Sequence error occur. \n\r" );
               HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_SER );
               HAL_SC_SetT34DR( ( SCn_Type* )SC0, 150, 150 );
               HAL_SC_AutoDeAct( ( SCn_Type* )SC0 );
               card_state = CARD_DEACTIVATION;
            }
            break;
         case CARD_WAIT_ATR:
            buffer_sc1[0] = DIRECT_CONV_DATA;
            HAL_SC_ControlConfig( ( SCn_Type* )SC1, SCn_CR1_RXE, DISABLE );
            HAL_SC_Transmit( ( SCn_Type* )SC1, buffer_sc1, 1, BLOCKING );

            intsrc = HAL_SC_GetStatus( ( SCn_Type* )SC0 );
            intsrc &= SCn_IFSR_BITMASK;

            // if Sequence or Convention Error Interrupt
            if( ( ( intsrc & SCn_IFSR_SER ) == SCn_IFSR_SER ) || ( ( intsrc & SCn_IFSR_CONER ) == SCn_IFSR_CONER ) )
            {
               _DBG( "The value of SC0->IFSR is " );
               _DBH32( intsrc );
               _DBG( ". Sequence or convention error occur. \n\r" );
               HAL_SC_SetT34DR( ( SCn_Type* )SC0, 150, 150 );
               HAL_SC_AutoDeAct( ( SCn_Type* )SC0 );
               card_state = CARD_DEACTIVATION;
               break;
            }

            // if Sequence or Convention End Interrupt
            if( ( ( intsrc & SCn_IFSR_SED ) == SCn_IFSR_SED ) && ( ( intsrc & SCn_IFSR_CONED ) == SCn_IFSR_CONED ) )
            {
               _DBG( "The value of SC0->IFSR is " );
               _DBH32( intsrc );
               _DBG( ". Sequence and convention finish. \n\r" );
               HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_ALL_EXCEPT_TXC );
               HAL_SC_ControlConfig( ( SCn_Type* )SC1, SCn_CR1_RXE, ENABLE );
               card_state = CARD_INTERFACE;
               break;
            }

            _DBG( "The value of SC0->IFSR is " );
            _DBH32( intsrc );
            _DBG( ". Sequence and convention finish. \n\r" );
            HAL_SC_SetT34DR( ( SCn_Type* )SC0, 150, 150 );
            HAL_SC_AutoDeAct( ( SCn_Type* )SC0 );
            card_state = CARD_DEACTIVATION;
            break;
         case CARD_DEACTIVATION:
            GPIO_ClrBit4( PD );
            _DBG( "The PD[4] outputs low level to finish SCI mode. \n\r" );
            delay();
            delay();
            HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_ALL_EXCEPT_TXC );
            HAL_SC_SetT34DR( ( SCn_Type* )SC0, 10000 - 1, 40000 - 1 );
            card_state = CARD_NONE;
            break;
         case CARD_INTERFACE:
            // receive data from PC, SC0, SC1
            len_pc = HAL_UART_Receive( ( UARTn_Type* )UART1, buffer_pc, sizeof( buffer_pc ), BLOCKING );
            len_sc0 = HAL_SC_Receive( ( SCn_Type* )SC0, buffer_sc0, sizeof( buffer_sc0 ), BLOCKING );
            if( ( HAL_SC_GetStatus( ( SCn_Type* )SC0 ) & SCn_IFSR_DOR ) == SCn_IFSR_DOR )
            {
               if( len_sc0 )
               {
                  len_sc0--;
               }
               HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_DOR );
            }

            len_sc1 = HAL_SC_Receive( ( SCn_Type* )SC1, buffer_sc1, sizeof( buffer_sc1 ), BLOCKING );
            if( ( HAL_SC_GetStatus( ( SCn_Type* )SC1 ) & SCn_IFSR_DOR ) == SCn_IFSR_DOR )
            {
               if( len_sc1 )
               {
                  len_sc1--;
               }
               HAL_SC_ClearStatus( ( SCn_Type* )SC1, SCn_STATUS_DOR );
            }

            // send the received data to SC1, SC0, PC
            if( len_pc > 0 )
            {
               HAL_SC_ControlConfig( ( SCn_Type* )SC0, SCn_CR1_RXE, DISABLE );
               HAL_SC_Transmit( ( SCn_Type* )SC0, buffer_pc, len_pc, BLOCKING );
               HAL_SC_ControlConfig( ( SCn_Type* )SC0, SCn_CR1_RXE, ENABLE );
            }

            if( len_sc0 > 0 )
            {
               HAL_UART_Transmit( ( UARTn_Type* )UART1, buffer_sc0, len_sc0, BLOCKING );
               if( buffer_sc0[len_sc0 - 1] == '\r' )
               {
                  buffer_sc0[0] = '\n';
                  HAL_UART_Transmit( ( UARTn_Type* )UART1, buffer_sc0, 1, BLOCKING );
               }
            }

            if( len_sc1 > 0 )
            {
               HAL_SC_ControlConfig( ( SCn_Type* )SC1, SCn_CR1_RXE, DISABLE );
               HAL_SC_Transmit( ( SCn_Type* )SC1, buffer_sc1, len_sc1, BLOCKING );
               HAL_SC_ControlConfig( ( SCn_Type* )SC1, SCn_CR1_RXE, ENABLE );
            }
            break;
         default:
            break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* SCIn_Configure */
   SCIn_Configure();

   /* SCn_SCI_PollingRun */
   SCn_SCI_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

