/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void UARTsc0_Configure( void );
void SCn_UART_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "SCn_UART_Polling Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: SC0 \n\r"
                        "Counterpart:       A external device like a PC \n\r"
                        "Tx Data:           Any character \n\r"
                        "UART TXD Pin:      PC4(SC0TXD) \n\r"
                        "UART RXD Pin:      PC3(SC0RXD) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         UARTsc0_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UARTsc0_Configure( void )
{
   SCn_CFG_Type      SCn_Config;

   // UARTsc0 configuration
   {
      // uart TXD/RXD pin configuration
      {
         // print
         _DBG( "PC4 was configured as a SC0TXD. \n\r" );
         _DBG( "PC3 was configured as a SC0RXD. \n\r\n\r" );

         // configure PC4 as a SC0TXD
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 4, ALTERN_FUNC );      // PC4            0: ----        1: ----        2: ----        3: MOSI1       4: SC0DATA     5: SC0TXD      6: CP1OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 4, AFSRx_AF5 );   // PC6   SWCLK    0: SWCLK       1: ----        2: TXD10       3: MOSI10      4: SC0DATA     5: SC0TXD      6: ----        7: ----

         // configure PC3 as a SC0RXD
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 3, ALTERN_FUNC );      // PC3            0: ----        1: ----        2: ----        3: MISO1       4: SC0RST      5: SC0RXD      6: CP0OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 3, AFSRx_AF5 );   // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24
      }

      // uart configuration
      {
         // get default SCn_Config for uart: 8-N-1
         HAL_SC_ConfigStructInit( &SCn_Config, SCn_UART_MODE, 38400 );

         // init SCn
         HAL_SC_Init( ( SCn_Type* )SC0, &SCn_Config );

         // enable SCn
         HAL_SC_Enable( ( SCn_Type* )SC0, ENABLE );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         SCn_UART_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SCn_UART_PollingRun( void )
{
   uint8_t     buf[100];   // tx/rx buffer
   uint32_t    len;

   /* Infinite loop */
   while( 1 )
   {
      // receive data from PC
      len = HAL_SC_Receive( ( SCn_Type* )SC0, buf, sizeof( buf ), BLOCKING );

      // send the received data to PC
      if( len > 0 )
      {
         HAL_SC_Transmit( ( SCn_Type* )SC0, buf, len, BLOCKING );
         if( buf[len - 1] == '\r' )
         {
            buf[0] = '\n';
            HAL_SC_Transmit( ( SCn_Type* )SC0, buf, 1, BLOCKING );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* UARTsc0_Configure */
   UARTsc0_Configure();

   /* SCn_UART_PollingRun */
   SCn_UART_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

