/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void TIMER40_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER40_Configure( void );
void TIMER4n_CaptureRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER4n_Capture Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:             TIMER40 \n\r"
                        "Capture Signal Generation Pin: PB0 \n\r"
                        "Timer Capture Input Pin:       PA0(T40INP) \n\r"
                        "Prerequisite:                  Connect PB0 and PA0(T40INP) with a jumper. \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    fflag = 0;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER40_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER40_IRQHandler_IT( void )
{
   uint32_t          status;

   // get interrupt flag
   status = T4nCapInt_GetFg( TIMER40 );

   // if Timer/Counter 4n Capture Interrupt
   if( status == 1 )
   {
      // clear interrupt flag
      T4nCapInt_ClrFg( TIMER40 );

      // update fflag
      if( fflag == 0 )
      {
         fflag = 1;
      }
      else if( fflag == 1 )
      {
         fflag = 2;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // capture signal generation pin configurtion
   {
      // print
      _DBG( "PB0 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER40_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER40_Configure( void )
{
   TIMER4n_CAPTURECFG_Type    TIMER4n_Config;

   // TIMER40 configuration
   {
      // timer capture input pin configuration
      {
         // print
         _DBG( "PA0 was configured as a T40INP. \n\r\n\r" );

         // configure PA0 as a T40INP
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 0, AFSRx_AF1 );
      }

      // timer periodic mode configuration
      {
         TIMER4n_Config.CkSel = TIMER4n_PCLK;               // PCLK = 32MHz
         TIMER4n_Config.Prescaler = ( 32 - 1 );             // fTIMER = PCLK / 32 = 1MHz
         TIMER4n_Config.PDR = ( 0xFFFF );                   // Period  = PDR / fTIMER
         TIMER4n_Config.ClrMode = TIMER4n_RISING_EGDE;
         TIMER4n_Config.ECE = TIMER4n_FALLING_EGDE;         // dummy parameter

         HAL_TIMER4n_Init( ( TIMER4n_Type* )TIMER40, TIMER4n_CAPTURE_MODE, &TIMER4n_Config );
         HAL_TIMER4n_ConfigInterrupt( ( TIMER4n_Type* )TIMER40, TIMER4n_INTCFG_CIE, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( TIMER40_IRQn, 3 );
      NVIC_EnableIRQ( TIMER40_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_TIMER40 );

      // enable timer (clear & start counter)
      HAL_TIMER4n_Cmd( ( TIMER4n_Type* )TIMER40, ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER4n_CaptureRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER4n_CaptureRun( void )
{
   uint32_t    i;
   uint32_t    count;

   // generate capture signal
   HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );   // low
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );     // high
   for( i = 0; i < 1000; i++ );                    // 2ms
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );   // low
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );     // high

   /* Infinite loop */
   while( 1 )
   {
      switch( fflag )
      {
         case 2:
            // updae fflag
            fflag = 0;

            // get capture data
            count = TIMER4n_GetCapData( TIMER40 );

            // print capture data
            _DBG( "Capture CNT = " );
            _DBD32( count );
            _DBG( "\n\r" );     // Capture data: 2071
            if( count == 2071 )
            {
               _DBG( "Capture CNT is Ok. \n\r" );
            }
            else
            {
               _DBG( "Capture CNT is not Ok. \n\r" );
            }
            break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER40_Configure */
   TIMER40_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* TIMER4n_CaptureRun */
   TIMER4n_CaptureRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

