/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void TIMER50_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER50_Configure( void );
void TIMER5n_CaptureRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER5n_Capture Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:             TIMER50 \n\r"
                        "Capture Signal Generation Pin: PB0 \n\r"
                        "Timer Capture Input Pin:       PF2(T50INP) \n\r"
                        "Prerequisite:                  Connect PB0 and PF2(T50INP) with a jumper. \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t          fflag = 0;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER50_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER50_IRQHandler_IT( void )
{
   uint32_t    status;

   // get interrupt flag
   status = TIMER50->CR_b.T5nCIFLAG;

   // if Timer/Counter 5n Capture Interrupt
   if( status == 1 )
   {
      // clear interrupt flag
      HAL_TIMER5n_ClearStatus( ( TIMER5n_Type* )TIMER50, TIMER5n_CR_T5nCIFLAG_Msk );

      // clear counter and prescaler
      HAL_TIMER5n_ClearCounter( ( TIMER5n_Type* )TIMER50 );

      // update fflag
      if( fflag == 0 )
      {
         fflag = 1;
      }
      else if( fflag == 1 )
      {
         fflag = 2;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // capture signal generation pin configurtion
   {
      // print
      _DBG( "PB0 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER50_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER50_Configure( void )
{
   TIMER5n_CFG_Type     TIMER5n_Config;

   // TIMER50 configuration
   {
      // timer capture input pin configuration
      {
         // print
         _DBG( "PF2 was configured as a T50INP. \n\r\n\r" );

         // configure PF2 as a T50INP
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 2, ALTERN_FUNC );      // PD4            0: ----        1: T50INP      2: RXD0        3: ----        4: SC1PWR      5: SDA0        6: CP1P5       7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 2, AFSRx_AF1 );   // PF2            0: ----        1: T50INP      2: ----        3: MISO0       4: AN14        5: ----        6: ----        7: SEG21
      }

      // timer capture mode configuration
      {
         // select peripheral clock: PCLK
         HAL_SCU_Peripheral_ClockSelection( PPCLKSR_T50CLK, T50CLK_PCLK );

         // init timer
         TIMER5n_Config.T5nMS = TIMER5n_CR_T5nMS_CaptureMode;
         TIMER5n_Config.T5nCLK = TIMER5n_CR_T5nCLK_IntPrescaledClock;   // PCLK = 32MHz
         TIMER5n_Config.T5nECE = TIMER5n_CR_T5nECE_FallingEdge;         // dummy parameter
         TIMER5n_Config.T5nINSEL = TIMER5n_CR_T5nINSEL_ExtClock;
         TIMER5n_Config.T5nINPOL = TIMER5n_CR_T5nINPOL_RisingEdge;
         TIMER5n_Config.T5nOPOL = TIMER5n_CR_T5nOPOL_StartHigh;         // dummy parameter
         TIMER5n_Config.ADR = 1000 - 1;                                 // Period  = ADR / fTIMER = 1000us
         TIMER5n_Config.BDR = 0;                                        // Duty(B) = BDR / fTIMER =    0us  // dummy parameter
         TIMER5n_Config.Prescaler = 32 - 1;                             // fTIMER = PCLK / 32 = 1MHz
         HAL_TIMER5n_Init( ( TIMER5n_Type* )TIMER50, &TIMER5n_Config );
         HAL_TIMER5n_ConfigInterrupt( ( TIMER5n_Type* )TIMER50, TIMER5n_CR_CAPTURE_INTR, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( TIMER50_IRQn, 3 );
      NVIC_EnableIRQ( TIMER50_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_TIMER50 );

      // enable timer (clear & start counter)
      HAL_TIMER5n_Cmd( ( TIMER5n_Type* )TIMER50, ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER5n_CaptureRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER5n_CaptureRun( void )
{
   uint32_t    i;
   uint32_t    count;

   // generate capture signal
   HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );   // low
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );     // high
   for( i = 0; i < 1000; i++ );                    // 1.51ms
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );   // low
   for( i = 0; i < 1000; i++ );                    // 248us
   HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );     // high

   /* Infinite loop */
   while( 1 )
   {
      switch( fflag )
      {
         case 2:
            // updae fflag
            fflag = 0;

            // get capture data
            count = HAL_TIMER5n_GetCaptureData( ( TIMER5n_Type* )TIMER50 );

            // print capture data
            _DBG( "Capture CNT = " );
            _DBD32( count );
            _DBG( "\n\r" );     // Capture data : 752
            if( count == 752 )
            {
               _DBG( "Capture CNT is Ok. \n\r" );
            }
            else
            {
               _DBG( "Capture CNT is not Ok. \n\r" );
            }
            break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER50_Configure */
   TIMER50_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* TIMER5n_CaptureRun */
   TIMER5n_CaptureRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

