/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void UART1_Configure( void );
void UARTn_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "UARTn_Polling Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: UART1 \n\r"
                        "Counterpart:       A external device like a PC \n\r"
                        "Tx Data:           Any character \n\r"
                        "UART TXD Pin:      PC10(TXD1) \n\r"
                        "UART RXD Pin:      PC11(RXD1) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART1_Configure( void )
{
   UARTn_CFG_Type    UARTn_Config;

   // UART1 configuration
   {
      // uart TXD/RXD pin configuration
      {
         // print
         _DBG( "PC10 was configured as a TXD1. \n\r" );
         _DBG( "PC11 was configured as a RXD1. \n\r\n\r" );

         // configure PC10 as a TXD1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 10, ALTERN_FUNC );   // PB0            0: ----        1: ----        2: TXD1        3: ----        4: AN10        5: ----        6: ----        7: SEG12
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 10, AFSRx_AF2 );   // PC10           0: ----        1: ----        2: TXD1        3: ----        4: SC0IN       5: ----        6: ----        7: COM6/SEG27

         // configure PC11 as a RXD1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 11, ALTERN_FUNC );   // PB1            0: ----        1: ----        2: RXD1        3: ----        4: AN11        5: LPDE        6: ----        7: SEG11
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 11, AFSRx_AF2 );   // PC11           0: ----        1: ----        2: RXD1        3: ----        4: ----        5: LPDE        6: ----        7: COM7/SEG28
         HAL_GPIO_ConfigPullup( ( Pn_Type* )PC, 11, PUPDx_EnablePU );
      }

      // uart configuration
      {
         // get default configuration for uart: 38400-8-N-1
         HAL_UART_ConfigStructInit( &UARTn_Config );

         // init UARTn
         HAL_UART_Init( ( UARTn_Type* )UART1, &UARTn_Config );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UARTn_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UARTn_PollingRun( void )
{
   uint8_t     buf[100];   // tx/rx buffer
   uint32_t    len;

   /* Infinite loop */
   while( 1 )
   {
      // receive data from PC
      len = HAL_UART_Receive( ( UARTn_Type* )UART1, buf, sizeof( buf ), BLOCKING );

      // send the received data to PC
      if( len > 0 )
      {
         HAL_UART_Transmit( ( UARTn_Type* )UART1, buf, len, BLOCKING );
         if( buf[len - 1] == '\r' )
         {
            buf[0] = '\n';
            HAL_UART_Transmit( ( UARTn_Type* )UART1, buf, 1, BLOCKING );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* UART1_Configure */
   UART1_Configure();

   /* UARTn_PollingRun */
   UARTn_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

