/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void SysTick_ExceptionHandler_IT( void );
void WDT_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void WDT_Configure( void );
void WDT_ResetRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "WDT_Reset Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: WDT \n\r"
                        "GPIO Output Pin:   PB0 ~ PB3 \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    msec;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         SysTick_ExceptionHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SysTick_ExceptionHandler_IT( void )
{
   if( msec > 0 )
   {
      msec--;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         WDT_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void WDT_IRQHandler_IT( void )
{
   uint32_t          status;
   static uint8_t    alternate_udf  = 1;     // toggle flag for underflow
   static uint8_t    alternate_wm   = 1;     // toggle flag for window match

   // get interrupt flag
   status = HAL_WDT_GetStatus();

   // if Watch-Dog Timer Underflow Interrupt
   if( status & 0x01 )
   {
      // clear interrupt flag
      HAL_WDT_ClearStatus( 1 );

      // toggle PB0 ~ PB1
      if( alternate_udf == 1 )
      {
         // clear flag
         alternate_udf = 0;

         // output high
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 1 ) );
      }
      else
      {
         // set flag
         alternate_udf = 1;

         // output low
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );
      }
   }

   // if Watch-Dog Timer Window Match Interrupt
   if( status & 0x02 )
   {
      // clear interrupt flag
      HAL_WDT_ClearStatus( 2 );

      // toggle PB2 ~ PB3
      if( alternate_wm == 1 )
      {
         // clear flag
         alternate_wm = 0;

         // output high
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 2 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 3 ) );
      }
      else
      {
         // set flag
         alternate_wm = 1;

         // output low
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PB0 ~ PB3 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );

      // configure PB1 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );

      // configure PB2 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 2, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );

      // configure PB3 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 3, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         WDT_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void WDT_Configure( void )
{
   // WDT configuration
   {
      // select peripheral clock: WDTRC
      HAL_SCU_Peripheral_ClockSelection( PPCLKSR_WDTCLK, WDTCLK_WDTRC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         WDT_ResetRun
 * @param         None
 * @return        None
 * @attention     IAR toolchain users should pay attention to the WDT Reset Time.
 *                If WDT Reset Time is too short, it is hard to debugging.
 *                So, we recomend to use the WDT Reset Time larger than 2 seconnds.
 *//*-------------------------------------------------------------------------*/
void WDT_ResetRun( void )
{
   WDT_CFG_Type      WDT_Config;

#if 1 // WDT_DR(0.5s) < WDTWDR(1s), clear in 900ms ==> Reset  (underflow)

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;       // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000 / 2;     // 0.5s
         WDT_Config.wdtWTmrConst = 10000;        // 1s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 900;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(0.5s) < WDTWDR(1s), clear in 400ms ==> NO reset 

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;       // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000 / 2;     // 0.5s
         WDT_Config.wdtWTmrConst = 10000;        // 1s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 400;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(1s) == WDTWDR(1s), clear in 900ms ==> NO reset 

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;    // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000;      // 1s
         WDT_Config.wdtWTmrConst = 10000;     // 1s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 900;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(1s) == WDTWDR(1s), clear in 1.2s ==> Reset (underflow)

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;    // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000;      // 1s
         WDT_Config.wdtWTmrConst = 10000;     // 1s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 1200;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(1s) > WDTWDR(0.5s), clear in 0.2  ==> Reset (window)

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;       // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000;         // 1s
         WDT_Config.wdtWTmrConst = 10000 / 2;    // 0.5s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 200;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(1s) > WDTWDR(0.5s), clear in 0.8s   ==> NO reset 

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;       // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000;         // 1s
         WDT_Config.wdtWTmrConst = 10000 / 2;    // 0.5s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 800;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* WDT_Configure */
   WDT_Configure();

   // configure SysTick to 1ms interrupt
   SysTick_Config( SystemCoreClock / 1000 );

   /* WDT_ResetRun */
   WDT_ResetRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

